#include <Servo.h>
#include <SoftwareSerial.h>

#define LASER 4
#define LOWER_SERVO 2
#define UPPER_SERVO 3
#define TRIG 11
#define ECHO 10
#define MYRX 8
#define MYTX 9

Servo downServo, upServo;
SoftwareSerial myserial(MYRX, MYTX);  // RX, TX

int input_int, i = 0, prev_nr, prev_prev_nr, start_away = 0;
int mode = 0, ok = 0, step = 1, period = 1, cycles = 0;
unsigned long previousMillis = 0, previousMillis2 = 0;

// for the ultrasound sensor
// const int trigPin = 11;
// const int echoPin = 10;
float duration, distance;

// function to execute auto mode
void autoMode(unsigned long time) {
  unsigned long currentMillis = millis();
  if ((unsigned long)(currentMillis - previousMillis) >= time + 1000 && cycles < 3 && cycles >= 0) {
    if (period == 1) {
      period++;
      upServo.write(90);
      downServo.write(90);
    } else if (period == 2) {
      period++;
      upServo.write(105);
      downServo.write(80);
    } else if (period == 3) {
      downServo.write(90);
      period++;
      upServo.write(90);
    } else if (period == 4) {
      downServo.write(105);
      period++;
      upServo.write(90);
    } else if (period == 5) {
      downServo.write(90);
      period++;
      upServo.write(90);
    } else if (period == 6) {
      downServo.write(80);
      period++;
      upServo.write(80);
    } else if (period == 7) {
      downServo.write(90);
      period++;
      upServo.write(90);
    } else {
      downServo.write(105);
      period = 1;
      upServo.write(90);
      cycles++;
    }

    previousMillis = currentMillis;
  }

  // after 3 cycles move to the right and slowly downwards for one second
  if ((unsigned long)(currentMillis - previousMillis) >= time + 500 && cycles == 3) {
    downServo.write(75);
    upServo.write(86);
    if (mode == 3) {
      cycles = -1;
    } else {
      cycles = 0;
    }

    previousMillis = currentMillis;
  }

  // if mode is 3, then stop movement and activate ultrasound sensor
  if ((unsigned long)(currentMillis - previousMillis) >= time + 1000 && mode == 3 && cycles == -1) {
    start_away = 0;
    cycles = 0;
    upServo.write(90);
    downServo.write(90);
    previousMillis = currentMillis;
  }
}


void ultrasonicSensor(unsigned long time) {
  unsigned long currentMillis = millis();
  if ((unsigned long)(currentMillis - previousMillis2) >= time) {
    digitalWrite(TRIG, LOW);
  }
}

void setup() {
  Serial.begin(9600);
  myserial.begin(9600);

  pinMode(LASER, OUTPUT);

  downServo.attach(LOWER_SERVO);
  upServo.attach(UPPER_SERVO);

  pinMode(TRIG, OUTPUT);
  pinMode(ECHO, INPUT);
}

void loop() {
  if (myserial.available()) {
    input_int = myserial.read();

    if (input_int == 254) {
      prev_nr = input_int;
    }

    if ((input_int == 1 || input_int == 2 || input_int == 3 || input_int == 4 || input_int == 5) && prev_nr == 254) {
      prev_prev_nr = prev_nr;
      prev_nr = input_int;
    }

    if (input_int == 0 && prev_nr == 1 && prev_prev_nr == 254) {
      mode = 1;
      ok = 0;
      i = 0;
      step = 1;
      period = 1;
      start_away = 0;
      cycles = 0;
      previousMillis = 0;
      Serial.println("set mode to 1");
    } else if (input_int == 0 && prev_nr == 2 && prev_prev_nr == 254) {
      mode = 2;
      ok = 0;
      i = 0;
      step = 1;
      period = 1;
      start_away = 0;
      cycles = 0;
      previousMillis = 0;
      Serial.println("setmode to 2");
    } else if (input_int == 0 && prev_nr == 3 && prev_prev_nr == 254) {
      mode = 3;
      ok = 0;
      i = 0;
      step = 1;
      period = 1;
      start_away = 0;
      cycles = 0;
      previousMillis = 0;
      Serial.println("set mode to 3");
    } else if (input_int == 0 && prev_nr == 4 && prev_prev_nr == 254) {
      ok = 0;
      i = 0;
      Serial.println("ON");
      digitalWrite(LASER, HIGH);
    } else if (input_int == 0 && prev_nr == 5 && prev_prev_nr == 254) {
      ok = 0;
      i = 0;
      Serial.println("OFF");
      digitalWrite(LASER, LOW);
    }


    if (mode == 2 && ok == 1) {
      // manual mode
      if (i == 0) {
        // lower servomotor movement
        if (input_int > 140) {
          downServo.write(80);
        } else if (input_int < 100) {
          downServo.write(105);
        } else {
          downServo.write(90);
        }
        Serial.print(input_int);
        Serial.print("  ");
        i++;
      } else if (i == 1) {
        // upper servomotor movement
        if (input_int > 150) {
          upServo.write(80);
        } else if (input_int < 80) {
          upServo.write(105);
        } else {
          upServo.write(90);
        }
        Serial.println(input_int);
        i = 0;
      }
    }

    ok = 1;
  }

  if (step == 1) {
    // stop servos after each mode switch
    upServo.write(90);
    downServo.write(90);
    step = 0;
  }

  if (mode == 1 && ok == 1) {
    // auto mode
    autoMode(1000);
  }

  if (mode == 3 && ok == 1) {
    // away mode
    if (start_away == 0) {
      upServo.write(90);
      downServo.write(90);
      digitalWrite(TRIG, HIGH);
      ultrasonicSensor(10);

      duration = pulseIn(ECHO, HIGH);
      distance = (duration * 0.0343) / 2;
      Serial.println(distance);
    }

    if (distance >= 10 && distance <= 15) {
      start_away = 1;
      autoMode(1000);
    }
  }
}
